@@ -1,7 +1,7 @@
 # Yahoo.pm
-# by Wm. L. Scheding and Martin Thurn
+# by Martin Thurn
 # Copyright (C) 1996-1998 by USC/ISI
-# $Id: Yahoo.pm,v 2.29 2003/01/23 14:04:02 mthurn Exp $
+# $Id: Yahoo.pm,v 2.32 2003-06-19 21:26:00-04 kingpin Exp kingpin $
 
 =head1 NAME
 
@@ -26,12 +26,7 @@ be done through L<WWW::Search> objects.
 
 =head1 NOTES
 
-The default search is: Yahoo's Inktomi-based index (not usenet).
-
-The default search is the "OR" of all query terms (not "AND").
-If you want the "AND" of all the query terms, add {'za' => 'and'} to the second argument to native_query.
-If you want to search the query as a phrase, add {'za' => 'phrase'} to the second argument to native_query.
-If you want to search the query as Yahoo's "intelligent default" (whatever that means), add {'za' => 'default'} to the second argument to native_query.
+The default search is: Yahoo's web-based index (not Directory).
 
 =head1 SEE ALSO
 
@@ -41,10 +36,6 @@ To make new back-ends, see L<WWW::Search>.
 
 Please tell the maintainer if you find any!
 
-=head1 TESTING
-
-This module adheres to the C<WWW::Search> test suite mechanism. 
-
 =head1 AUTHOR
 
 As of 1998-02-02, C<WWW::Search::Yahoo> is maintained by Martin Thurn
@@ -61,67 +52,7 @@ MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 
 =head1 VERSION HISTORY
 
-If it''s not listed here, then it wasn''t a meaningful nor released revision.
-
-=head2 2.29, 2003-01-23
-
-FIX parsing for new yahoo.com page format
-
-=head2 2.28, 2002-11-08
-
-FIX pattern for single-digit result-count integer match
-
-=head2 2.27, 2002-10-31
-
-new URL; tweak parser for new webpage format
-
-=head2 2.24, 2001-12-24
-
-fix for slightly changed output format
-
-=head2 2.23, 2001-07-16
-
-even better support for subclassing
-
-=head2 2.22, 2001-03-31
-
-added support for subclassing, for regional Yahoo sites
-
-=head2 2.21, 2000-12-15
-
-clean up URL parsing (yahoo.com added text to it)
-
-=head2 2.19, 2000-11-10
-
-rewrote parser using HTML::TreeBuilder
-
-=head2 2.18, 2000-10-02
-
-fixed parsing again.
-
-=head2 2.17, 2000-09-22
-
-fix description parsing and URL parsing
-
-=head2 2.16, 2000-09-19
-
-fix gui-style results parsing & new URL
-
-=head2 2.15, 2000-09-14
-
-fix result-count parsing
-
-=head2 2.14, 2000-07-05
-
-output format changed (thanks to Bin Yu for fixes)
-
-=head2 2.11, 2000-04-27
-
-new URL for gui_query
-
-=head2 2.09, 2000-03-27
-
-fixed for new CGI options
+See ChangeLog for all changes since version 2.07
 
 =head2 2.07, 2000-02-04
 
@@ -174,7 +105,7 @@ package WWW::Search::Yahoo;
 # @EXPORT_OK = qw();
 @ISA = qw( WWW::Search ); # Exporter);
 
-$VERSION = sprintf("%d.%02d", q$Revision: 2.29 $ =~ /(\d+)\.(\d+)/o);
+$VERSION = sprintf("%d.%02d", q$Revision: 2.32 $ =~ /(\d+)\.(\d+)/o);
 $MAINTAINER = 'Martin Thurn <mthurn@cpan.org>';
 
 use Carp ();
@@ -193,9 +124,10 @@ sub gui_query
                          'p' => $sQuery,
                          # 'hc' => 0,
                          # 'hs' => 0,
+                         'ei' => 'UTF-8',
                         };
   # print STDERR " +   Yahoo::gui_query() is calling native_query()...\n";
-  $rh->{'search_base_url'} = 'http://google.yahoo.com';
+  $rh->{'search_base_url'} = 'http://search.yahoo.com';
   $rh->{'search_base_path'} = '/bin/query';
   return $self->native_query($sQuery, $rh);
   } # gui_query
@@ -207,14 +139,14 @@ sub native_setup_search
   # print STDERR " +     This is Yahoo::native_setup_search()...\n";
   # print STDERR " +       _options is ", $self->{'_options'}, "\n";
 
-  # As of 2000-03-27 or so, yahoo.com does NOT let you choose the
-  # number of hits per page:
-  $self->{'_hits_per_page'} = 20;
+  $self->{'_hits_per_page'} = 100;
+  # $self->{'_hits_per_page'} = 10;  # for debugging
 
-  # If we run as a robot, WWW::RobotRules fetches the
-  # http://www.yahoo.com instead of http://www.yahoo.com/robots.txt,
-  # and dumps a thousand warnings to STDERR.
+  # www.yahoo.com refuses robots.
   $self->user_agent('non-robot');
+  # www.yahoo.com completely changes the HTML output depending on the
+  # browser!
+  # $self->{'agent_name'} = 'Mozilla/4.0 (compatible; MSIE 5.5; Windows 98)';
   # $self->{agent_e_mail} = 'mthurn@cpan.org';
 
   $self->{_next_to_retrieve} = 1;
@@ -228,7 +160,7 @@ sub native_setup_search
     $self->{'_options'} = {
                            'vo' => $native_query,
                            'h' => 'w',  # web sites
-                           # 'n' => $self->{_hits_per_page},
+                           'n' => $self->{_hits_per_page},
                            # 'b' => $self->{_next_to_retrieve}-1,
                           };
     } # if
@@ -262,6 +194,20 @@ sub native_setup_search
   } # native_setup_search
 
 
+sub preprocess_results_page
+  {
+  my $self = shift;
+  my $sPage = shift;
+  # Delete the <BASE> tag that appears BEFORE the <html> tag (because
+  # it causes HTML::TreeBuilder to NOT be able to parse it!)
+  $sPage =~ s!<BASE\s[^>]+>!!;
+  return $sPage;
+  # For debugging only.  Print the page contents and abort.
+  print STDERR $sPage;
+  exit 88;
+  } # preprocess_results_page
+
+
 sub parse_tree
   {
   my $self = shift;
@@ -271,80 +217,84 @@ sub parse_tree
   my $iCountSpoof = 0;
   my $WS = q{[\t\r\n\240\ ]};
   # Only try to parse the hit count if we haven't done so already:
-  if ($self->approximate_hit_count < 1)
+  print STDERR " + start, approx_h_c is ==", $self->approximate_hit_count(), "==\n" if 2 <= $self->{_debug};
+  if ($self->approximate_hit_count() < 1)
     {
-    # The hit count is inside a <FONT> tag:
-    my @aoFONT = $tree->look_down('_tag', 'font');
- FONT_TAG:
-    foreach my $oFONT (@aoFONT)
+    # Sometimes the hit count is inside a <DIV> tag:
+    my @aoDIV = $tree->look_down('_tag' => 'div',
+                                  'class' => 'ygbody',
+                                 );
+ DIV_TAG:
+    foreach my $oDIV (@aoDIV)
       {
-      my $s = $oFONT->as_text;
-      print STDERR " + try FONT == ", $oFONT->as_HTML if 2 <= $self->{_debug};
-      # print STDERR " +   TEXT == ", $s, "\n" if 2 <= $self->{_debug};
-      if ($s =~ m!(\d+)\s*-\s*(\d+)\s+of\s+([,0-9]+)!i)
+      next unless ref $oDIV;
+      print STDERR " + try DIV ==", $oDIV->as_HTML if 2 <= $self->{_debug};
+      my $s = $oDIV->as_text;
+      print STDERR " +   TEXT ==$s==\n" if 2 <= $self->{_debug};
+      if ($s =~ m!out\s+of\s+(?:about\s+)?([,0-9]+)!i)
         {
-        my ($iStart, $iPageMax, $iCount) = ($1, $2, $3);
-        # Looks like a page range and hit count, but we have to make
-        # sure we're looking at the right section of output:
-        my $oTDmom = $oFONT->parent;
-        next FONT_TAG unless ref $oTDmom;
-        my $oTDaunt = $oTDmom->left;
-        next FONT_TAG unless ref $oTDaunt;
-        my $sSection = $oTDaunt->as_text;
-        next FONT_TAG unless ($sSection =~ m!Web Matches!i);
-        print STDERR " +   found numbers $iStart $iPageMax $iCount\n" if 2 <= $self->{_debug};
+        my $iCount = $1;
         $iCount =~ s!,!!g;
         $self->approximate_result_count($iCount);
-        if (
-            # Last hit on this page is NOT a multiple of ten
-            # (i.e. looks like this is the last page of results)...
-            (($iPageMax % 10) ne 0)
-            &&
-            # BUT last hit on this page is NOT the last global hit!?!
-            ($iPageMax < $iCount))
-          {
-          # Yahoo.com is trying to trick us???
-          $iCountSpoof = 1;
-          } # if
-        last FONT_TAG;
+        last DIV_TAG;
         } # if
-      } # foreach
+      } # foreach DIV_TAG
     } # if
-
-  # Each URL result is in a <LI> tag:
-  my @aoLI = $tree->look_down('_tag', 'li');
- LI:
-  foreach my $oLI (@aoLI)
+  if ($self->approximate_hit_count() < 1)
     {
-    printf STDERR " + LI == %s\n", $oLI->as_HTML if 2 <= $self->{_debug};
-    #  + LI == <li><p><font face="arial"><big> <a href="http://srd.yahoo.com/goo/%22Shelagh+Fraser%22/20/T=1009210773/F=191de59def3b5fd43b17abdedd4397ad/*http://home.fuse.net/mckee/addresses.htm"> Star Wars Stars' Addresses</a> </big><br><b>...</b> Denis Lawson c/o Star Wars Fan Club PO Box 111000 Aurora, CO 80042. Aunt Beru<br><b>Shelagh</b> <b>Fraser</b> c/o Ken McReddie Ltd. 91 Regent St. London W1R 7TB, ENGLAND <b>...</b><br><font color=006600>http://home.fuse.net/mckee/addresses.htm</font> </font><p><table><tr><td height=4></td></tr></table><table bgcolor="e3e9f8" border=0 cellpadding=2 cellspacing=0 width="100%"><tr><td align="right" nowrap><font face="arial" size="-1">1-20 of 318 |&nbsp;<b><a href="http://google.yahoo.com/bin/query?\np=%22Shelagh+Fraser%22&amp;b=21&amp;hc=0&amp;hs=0&amp;xargs=">Next 20 &gt;</a></b> </font></td></tr></table><table><tr><td height=6></td></tr></table>
+    # Sometimes the hit count is inside a <small> tag:
+    my @aoDIV = $tree->look_down('_tag' => 'small',
+                                 );
+ SMALL_TAG:
+    foreach my $oDIV (@aoDIV)
+      {
+      next unless ref $oDIV;
+      print STDERR " + try SMALL ==", $oDIV->as_HTML if 2 <= $self->{_debug};
+      my $s = $oDIV->as_text;
+      print STDERR " +   TEXT ==$s==\n" if 2 <= $self->{_debug};
+      if ($s =~ m!out\s+of\s+(?:about\s+)?([,0-9]+)!i)
+        {
+        my $iCount = $1;
+        $iCount =~ s!,!!g;
+        $self->approximate_result_count($iCount);
+        last SMALL_TAG;
+        } # if
+      } # foreach DIV_TAG
+    } # if
+  print STDERR " + found approx_h_c is ==", $self->approximate_hit_count(), "==\n" if 2 <= $self->{_debug};
 
-    # The first <a> tag contains the title:
-    my $oA = $oLI->look_down('_tag', 'a');
-    next LI unless ref($oA);
-    my $sTitle = $oA->as_text;
+  # Unfortunately, the URL results are not logically marked up with
+  # HTML.  We have to resort to old-fashioned string parsing!
+  my $sAll = $tree->as_HTML;
+  my @asChunk = split('<big>', $sAll);
+  # Throw out what's before the first <big> tag:
+  shift @asChunk;
+ CHUNK:
+  foreach my $sChunk (@asChunk)
+    {
+    # The last chunk ends with </span>:
+    $sChunk =~ s!</span>.*\Z!!;
+    print STDERR " +   consider <big> chunk ==$sChunk==\n" if 2 <= $self->{_debug};
+    # The first <A> tag contains the URL and title:
+    unless ($sChunk =~ s!\A.*?<a\shref="([^"]+)">(.+?)</a>!!)
+      {
+      print STDERR " --- did not find <A> inside <big> chunk\n" if 2 <= $self->{_debug};
+      next CHUNK;
+      } # unless
+    my ($sURL, $sTitle) = ($1, $2);
     print STDERR " +   TITLE == $sTitle\n" if 2 <= $self->{_debug};
-    my $sURL = $oA->attr('href');
-    $oA->detach();
-    $oA->delete();
-    $sURL =~ s!^.+?\*!!;
+    # Delete Yahoo-redirect portion of URL:
+    next CHUNK unless ($sURL =~ s!\A.+?\*-!!);
     print STDERR " +   URL   == $sURL\n" if 2 <= $self->{_debug};
     # Ignore Yahoo Directory categories, etc.:
-    next LI if $sURL =~ m!(\A|/search/empty/\?)http://dir\.yahoo\.com!;
-    # Delete the FONT tag:
-    my $oFONT = $oLI->look_down('_tag' => 'font');
-    if (ref $oFONT)
-      {
-      $oFONT->detach();
-      $oFONT->delete();
-      } # if
+    next CHUNK if $sURL =~ m!(\A|/search/empty/\?)http://dir\.yahoo\.com!;
+    # Delete all remaining <A> tags:
+    $sChunk =~ s!<a\s.+?</a>!!g;
     # The remaining text of the LI is the description:
-    my $sDesc = $oLI->as_text || '';
+    my $sDesc = $sChunk;
+    print STDERR " +   raw DESC  ==$sDesc==\n" if 2 <= $self->{_debug};
     # Chop off extraneous:
-    $sDesc =~ s!\A$WS*-$WS+!!;
-    $sDesc =~ s!More\s+(Results|sites)\s+(about|From):.*?\Z!!i;
-    # $sDesc =~ s!\s?\d+-\d+\sof\s\d+\s\|?\240.*?\Z!!i;
-
+    $sDesc =~ s!\s+\|\s+.*?\Z!!i;
     print STDERR " +   DESC  == $sDesc\n" if 2 <= $self->{_debug};
     my $hit = new WWW::SearchResult;
     $hit->add_url($sURL);
@@ -354,28 +304,27 @@ sub parse_tree
     $hit->description($sDesc);
     push(@{$self->{cache}}, $hit);
     $hits_found++;
-    $oLI->delete;
-    $oLI->detach;
-    } # foreach oLI
+    } # foreach CHUNK
 
   if (! $iCountSpoof)
     {
     # The "next" button is in a table...
-    my @aoTABLE = $tree->look_down('_tag', 'table');
+    my @aoTABLE = $tree->look_down('_tag' => 'table');
     # ...but we want the LAST one appearing on the page:
  TABLE:
     foreach my $oTABLE (reverse @aoTABLE)
       {
       printf STDERR " + TABLE == %s\n", $oTABLE->as_HTML if 2 <= $self->{_debug};
-      my @aoA = $oTABLE->look_down('_tag', 'a');
+      my @aoA = $oTABLE->look_down('_tag' => 'a');
  A:
       foreach my $oA (@aoA)
         {
         printf STDERR " +   A == %s\n", $oA->as_HTML if 2 <= $self->{_debug};
-        if ($oA->as_text =~ m!Next\s+\d+!i)
+        if ($oA->as_text =~ m!\ANext(\s+\d+)?\Z!i)
           {
           my $sURL = $oA->attr('href');
-          $sURL =~ tr!\r\n!!d;
+          # Delete Yahoo-redirect portion of URL:
+          $sURL =~ s!\A.+?\*-!!;
           $self->{_next_url} = $self->absurl($self->{'_prev_url'}, $sURL);
           last TABLE;
           } # if
@@ -389,7 +338,7 @@ sub parse_tree
 sub strip
   {
   my $self = shift;
-  my $s = shift;
+  my $s = &WWW::Search::strip_tags(shift);
   $s =~ s!\A[\240\t\r\n\ ]+  !!x;
   $s =~ s!  [\240\t\r\n\ ]+\Z!!x;
   return $s;
