@@ -1,5 +1,5 @@
 
-# $Id: Advanced.pm,v 1.7 2002/11/01 15:08:21 mthurn Exp $
+# $Id: Advanced.pm,v 1.8 2003-06-19 21:26:55-04 kingpin Exp kingpin $
 
 =head1 NAME
 
@@ -70,6 +70,10 @@ MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 
 If it''s not listed here, then it wasn''t a meaningful nor released revision.
 
+=head2 2.05, 2003-05-30
+
+overhaul for new webpage format
+
 =head2 2.04, 2002-10-31
 
 overhaul for new webpage format
@@ -94,7 +98,7 @@ use strict;
 use vars qw( @ISA $VERSION $MAINTAINER );
 @ISA = qw( WWW::Search::Yahoo );
 
-$VERSION = '2.04';
+$VERSION = '2.05';
 $MAINTAINER = 'Martin Thurn <mthurn@cpan.org>';
 
 sub native_setup_search
@@ -137,7 +141,7 @@ sub native_setup_search
                          # 's' => '-s',  # sort order
                         };
   $rh->{'search_base_url'} = 'http://search.news.yahoo.com';
-  $rh->{'search_base_path'} = '/search/news';
+  $rh->{'search_base_path'} = '/search/news/';
   # print STDERR " +   Yahoo::UK::native_setup_search() is calling SUPER::native_setup_search()...\n";
   return $self->SUPER::native_setup_search($sQuery, $rh);
   } # native_setup_search
@@ -146,14 +150,114 @@ sub native_setup_search
 sub preprocess_results_page
   {
   my $self = shift;
-  # Confound it!  All the results are in a <P> except the first one!
   my $s = shift;
-  $s =~ s!</font></td></tr></table>!</table> <P>!g;
+  # Remove all carriage-returns:
+  $s =~ tr!\r\n!!d;
+  # Convert nbsp to plain space:
+  $s =~ s!&nbsp;! !g;
+  # Delete bold tags which appear around the query terms in the descriptions:
+  $s =~ s!</?b>!!gi;
+  # Insert carriage-return before every HTML tag:
+  $s =~ s!(</?\w)!\n$1!g;
+  # Insert carriage-return after every HTML tag:
+  $s =~ s!(\S>)!$1\n!g;
+  # Delete blank lines:
+  $s =~ s!\n\s*\n!\n!g;
+  $s =~ s!\n\s*\n!\n!g;
+  if (0 && ($s =~ m!tmpl=story!))
+    {
+    print STDERR $s;
+    exit 9;
+    } # if
   return $s;
   } # preprocess_results_page
 
 
-sub parse_tree
+sub native_retrieve_some
+  {
+  my $self = shift;
+  # printf STDERR (" +   %s::native_retrieve_some()\n", __PACKAGE__) if $self->{_debug};
+  # fast exit if already done
+  return undef if (!defined($self->{_next_url}));
+  # If this is not the first page of results, sleep so as to not overload the server:
+  $self->user_agent_delay if 1 < $self->{'_next_to_retrieve'};
+  # Get one page of results:
+  print STDERR " +   submitting URL (", $self->{'_next_url'}, ")\n" if $self->{_debug};
+  my $response = $self->http_request($self->http_method, $self->{'_next_url'});
+  print STDERR " +     got response\n", $response->headers->as_string, "\n" if 2 <= $self->{_debug};
+  $self->{_prev_url} = $self->{_next_url};
+  # Assume there are no more results, unless we find out otherwise
+  # when we parse the html:
+  $self->{_next_url} = undef;
+  $self->{response} = $response;
+  # print STDERR " --- HTTP response is:\n", $response->as_string if 4 < $self->{_debug};
+  if (! $response->is_success)
+    {
+    if ($self->{_debug})
+      {
+      print STDERR " --- HTTP request failed, response is:\n", $response->as_string;
+      } # if
+    return undef;
+    } # if
+  # Pre-process the output:
+  my $sPage = $self->preprocess_results_page($response->content);
+  # ABOVE WAS COPIED FROM WWW::Search::native_retrieve_some()
+  # Parse the output:
+  my $hits_found = 0;
+  my @asLine = $self->split_lines($sPage);
+  chomp @asLine;
+ LINE:
+  while (defined(my $sLine = shift @asLine))
+    {
+    if (($self->approximate_result_count == 0)
+        &&
+        ($sLine =~ m!\A\s*\d+\s*-\s*\d+\s+of\s+(\d+)!))
+      {
+      my $iCount = $1;
+      # print STDERR " +   found number $iCount\n" if 2 <= $self->{_debug};
+      $self->approximate_result_count($iCount);
+      next LINE;
+      } # if
+    next LINE unless ($sLine =~ m!<a href="(.+tmpl=story.+)">!);
+    my $sURL = $1;
+    print STDERR " +   found url ==$sURL==\n" if 2 <= $self->{_debug};
+    my $sTitle = shift @asLine;
+    print STDERR " +   found title ==$sTitle==\n" if 2 <= $self->{_debug};
+    my $sDate;
+    if ($self->lookfor('</span>', \@asLine))
+      {
+      $sDate = shift @asLine;
+      } # if
+    print STDERR " +   found date ==$sDate==\n" if 2 <= $self->{_debug};
+    my $sDesc = shift @asLine;
+    $sDesc = shift @asLine;
+    print STDERR " +   found description ==$sDesc==\n" if 2 <= $self->{_debug};
+    my $hit = new WWW::Search::Result;
+    $hit->add_url($sURL);
+    $hit->title($sTitle);
+    $hit->description($sDesc);
+    $hit->change_date($sDate);
+    push(@{$self->{cache}}, $hit);
+    $self->{'_num_hits'}++;
+    $hits_found++;
+    } # foreach
+  return $hits_found;
+  } # native_retrieve_some
+
+sub lookfor
+  {
+  my $self = shift;
+  my ($sPattern, $ras) = @_;
+  while (defined(my $s = shift @$ras))
+    {
+    return $s if ($s =~ m!$sPattern!);
+    } # while
+  # Ran off end of array?
+  return undef;
+  } # lookfor
+
+
+sub parse_tree_UNUSED
   {
   my $self = shift;
   my $tree = shift;
@@ -294,3 +398,35 @@ sub skip_text_elements
 __END__
 
 http://search.news.yahoo.com/search/news?p=george+lucas&s=&n=10&o=&2=&3=05/05/01-05/15/01
+
+Actual pre-processed result:
+
+<a href="http://story.news.yahoo.com/news?tmpl=story&u=/ap/20030522/ap_on_re_as/everest_50th_anniversary_11">
+Dozens Head to Mount Everest's Summit
+</a>
+<a href="http://story.news.yahoo.com/news?tmpl=story&u=/ap/20030522/ap_on_re_as/everest_50th_anniversary_11" target=awindow>
+<img src="http://us.i1.yimg.com/us.yimg.com/i/us/search/bn/newwin_1.gif" height="11" width="11" border="0" align="middle" vspace="1" hspace="4" alt="Open this result in new window">
+</a>
+<br>
+<div class= timedate >
+<span class=provtimedate>
+ AP  - 
+</span>
+ May 22  9:25 AM 
+</div>
+...Miura, a native of the northern Japanese city of Aomori, began his skiing career in 1962 and won acclaim eight years later by becoming the first person...
+<br>
+In 
+<a href="http://news.yahoo.com/">
+Yahoo! News
+</a>
+ > 
+<a href="http://news.yahoo.com/news?tmpl=index2&cid=721">
+World
+</a>
+ > 
+<a href="http://news.yahoo.com/news?tmpl=index2&cid=516">
+AP
+</a>
+<br>
+<p>
